
#include "wifi.h"

int num_char;					//number of characters in the string of UART1
int num_send_chars;				
static unsigned char character;	//the incoming character in the string

/**************************************************************************/
/*!
    @brief  Child thread to read in uart data
*/
/**************************************************************************/
int PT_Get_WiFi(struct pt *pt)
{
    // mark the beginnning of the input thread
    PT_BEGIN(pt);
    
    num_char = 0;
    //memset(term_buffer, 0, max_chars);

    while(num_char < MAXWIFILINELENGTH)
    {
        if(UART1GetErrors() & 0x02){
            UART1ClearAllErrors();
            PT_YIELD_TIME_msec(100);
            break;
        }
        // get the character
        // yield until there is a valid character so that other
        // threads can execute
        int time = PT_GET_TIME();
        //PT_YIELD_UNTIL(pt, (PT_GET_TIME() > time + 100));
        PT_YIELD_UNTIL(pt, UARTReceivedDataIsAvailable(UART1));
        //if (!UARTReceivedDataIsAvailable(UART1)) {
            //break;
        //}
        character = UARTGetDataByte(UART1);

        //if the incoming character is '\n', that means the end of the string
        if(character == '\n'){        
            PT_wifi_buffer[num_char] = 0; // zero terminate the string
            break;
        }
        
        else  {PT_wifi_buffer[num_char++] = character ;}
        
    } //end while(num_char < max_size)   

    // kill this input thread, to allow spawning thread to execute
    PT_EXIT(pt);
    // and indicate the end of the thread
    PT_END(pt);
}

/**************************************************************************/
/*!
    @brief  Initialize wifi communication
*/
/**************************************************************************/
void wifi_init(void) {
    PPSInput (3, U1RX, RPB2);
    PPSOutput(1, RPB3, U1TX);
    UARTConfigure(UART1, UART_ENABLE_PINS_TX_RX_ONLY);
    UARTSetLineControl(UART1, UART_DATA_SIZE_8_BITS | UART_PARITY_NONE | UART_STOP_BITS_1);
    UARTSetDataRate(UART1, pb_clock, BAUDRATE);
    UARTEnable(UART1, UART_ENABLE_FLAGS(UART_PERIPHERAL | UART_RX | UART_TX));
    
    phone_latitude = phone_longitude = phone_altitude = 0.0; // float
    flight = kill;
}

/**************************************************************************/
/*!
    @brief  Parse a wifi message
*/
/**************************************************************************/
int parse_wifi(char *s) {
    // look for a few common sentences
    if (strstr(s, "$GPS")) {
        // found GPS
        char *p = s;

        // parse out latitude
        p = strchr(p, ',')+1;
        float latitude_buff = 0.0;
        char* lat_out;
        latitude_buff = strtof (p, &lat_out);
        
        // parse out longitude
        float longitude_buff = 0.0;
        char* lon_out;
        longitude_buff = strtof (lat_out+1, &lon_out);
        
        // parse out altitude
        float altitude_buff = 0.0;
        char* alt_out;
        altitude_buff = strtof (lon_out+1, &alt_out);
        
        phone_latitude = latitude_buff;
        phone_longitude = longitude_buff;
        phone_altitude = altitude_buff;
    }
  
    else if (strstr(s, "$CONTROL")) {
        // found CONTROL
        if (strstr(s, "Take Off")) {
            flight = take_off;
            take_off_start_time = PT_GET_TIME();
        }
        else if (strstr(s, "Land")) {
            flight = land;
        }
        else if (strstr(s, "Stop")) {
            flight = kill;
        }
    }
  
    else if (strstr(s, "$CONNECTION")) {
        // found CONNNECTION
        if (strstr(s, "failure")) {
            connection = failure;
        }
        else if (strstr(s, "success")) {
            connection = success;
        }
    }
}

/**************************************************************************/
/*!
    @brief  Parse a hex character
*/
/**************************************************************************/
char parseHex_wifi(char c) {
    if (c < '0')
      return 0;
    if (c <= '9')
      return c - '0';
    if (c < 'A')
       return 0;
    if (c <= 'F')
       return (c - 'A')+10;
    // if (c > 'F')
    return 0;
}
